class CollectionAlreadyExistsError(Exception): ...
class CollectionNotFoundError(Exception): ...
class CollectionValidationError(Exception): ...
class DocumentValidationError(Exception): ...
class InvalidArgumentError(Exception): ...
class PermissionDeniedError(Exception): ...
class QueryLsnTimeoutError(Exception): ...
class QuotaExceededError(Exception): ...
class RequestTooLargeError(Exception): ...
class SchemaValidationError(Exception): ...
class SlowDownError(Exception): ...
